/**********************************************************************************************

Author : Mauro Laurenti
Version : 1.1
Date : 4/9/2006
Last Update: 09/03/2012
 
CopyRight 2006-2012 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) 
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).  
The software is owned by the Author, and is protected under applicable copyright laws. 
All rights are reserved. 
Any use in violation of the foregoing restrictions may subject the 
user to criminal sanctions under applicable laws (Italian or International ones), as well as to 
civil liability for the breach of the terms and conditions of this license. 
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following 
software doesn't aim to commercial use or monetary earning of any kind.    

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT, 
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


 This library contains all the functions that are handy for controlling 			
 an LCD with a 44780 Hitachi controller.							
 	
 To use these functions for your pourpose you must set up properly 
 the LCD constants and the pins of the PORT/s that you will use.
 You can use any pin you want!

**********************************************************************************************/

#include <p18cxxx.h>
#include <delay.h>
#include <ctype.h>


#ifndef FLAG_LCD_44780
#define FLAG_LCD_44780

// Prototipe for itoa from ctype lib 
char *itoa (int value, char *s);

//**************************************************
// LCD constants
// All the following pins must be set as output
//**************************************************

/**
 * This value are used as default ones
 *
 */
#ifndef LCD_DEFAULT

	#warning LCD_D0 has been not defined, LATDbits.LATD4 will be used
	#warning LCD_D1 has been not defined, LATDbits.LATD5 will be used
	#warning LCD_D2 has been not defined, LATDbits.LATD6 will be used
	#warning LCD_D3 has been not defined, LATDbits.LATD7 will be used
	#warning LCD_RS has been not defined, LATDbits.LATD2 will be used
	#warning LCD_E has been not defined, LATDbits.LATD3 will be used
	#warning LCD_RW has been not defined, LATDbits.LATD1 will be used
	#warning LCD_LED has been not defined, LATCbits.LATC1 will be used
	
#endif

	#define LCD_D0 LATDbits.LATD4
	#define LCD_D1 LATDbits.LATD5
	#define LCD_D2 LATDbits.LATD6	
	#define LCD_D3 LATDbits.LATD7	
	#define LCD_RS LATDbits.LATD2
	#define LCD_E LATDbits.LATD3	
	#define LCD_RW LATDbits.LATD1	
	#define LCD_LED LATCbits.LATC1	
	
	


//**************************************************
//               Constant Definitions

#define LEFT 0
#define RIGHT 1

#define TURN_ON_LED 1
#define TURN_OFF_LED 0

#define TURN_ON_CURSOR 1
#define TURN_OFF_CURSOR 0

#define BLINK_ON 1
#define BLINK_OFF 0

//**************************************************


/**
 * This function generates the Enable pulse.
 *
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void Epulse (void);


/**
 * This function generates the Enable pulse.
 *
 *
 * @param bit_0 bit 0 of the data bus (4 bit modality)
 *
 * @param bit_1 bit 1 of the data bus (4 bit modality)
 *
 * @param bit_2 bit 2 of the data bus (4 bit modality)
 *
 * @param bit_3 bit 3 of the data bus (4 bit modality)
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void SendCommand (unsigned char bit_3, unsigned char bit_2, unsigned char bit_1, unsigned char bit_0);


/**
 * This function locates the cursor at the beginning of Line 2.
 *
 */
void Line2LCD(void);


/**
 * This function locates the cursor at home location. First line first character.
 *
 */
void HomeLCD(void);


/**
 * This function shifts the LCD screen on the left or rigt.
 *
 * @param shift Specify where the shift should be [LEFT, RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void ShiftLCD(char shift, char number_of_shift);


/**
 * This function shifts the LCD cursor on the left or rigt. The position of the cursor is where the
 * next writting will be performed.
 *
 * @param shift Specify where the shift should be [LEFT, RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void ShiftCursorLCD(char shift, char number_of_shift);


/**
 * This function locates the LCD cursor on the selected line. Tested on 20x4 16x2 LCD displays.
 *
 * @param line Specify the number of the line [1,2,3,4]
 *
 * @warning It might not work with all the LCD Dipslays.
 */
void GotoLineLCD (char line);


/**
 * This function writes a char to the LCD display. (e.g WriteCharLCD ('a'); )
 *
 * @param value Specify the character to be sent [0-128 ASCII code].
 *
 * @note The function accept ASCII integer or constants 'a'..'z'.
 */
void WriteCharLCD (unsigned char value);


/**
 * This function writes a const string to the LCD display, e.g WriteStringLCD("Hello"); .
 *
 * @param buffer Is a const string written in rom (is not an array).
 *
 * @note The function accepts strings written in rom (constant), it does not accept array.
 *       To write array refers the WriteVarLCD () function.
 *
 */
void WriteStringLCD(const rom char *buffer);


/**
 * This function writes an array of char to the LCD display, terminated with /0 .
 *
 * @param buffer It is an array of char terminated with /0 .
 *
 * @note The function accepts strings  within an array. To use const array within rom use WriteStringLCD ().
 *
 */
void WriteVarLCD(unsigned char *buffer);


/**
 * This function writes an integer to the LCD display. The integer is converted to string.
 *
 * @param value It is integer that must be written to the LCD diplay.
 *
 * @param numeber_of_digits It specifies the number of shown digit [0-5].
*                    0: Left Justified
*                    1-5: Right Justified with n digit
 *
 * @note If you set a number of digit less than required the digit will be lost starting from the less
 *       significant digit. Minus is like a digit.
 *
 */
void WriteIntLCD(int value, char number_of_digits);


/**
 * This Function cleans the LCD display.
 *
 */
void ClearLCD (void);


/**
 * This function controls the cursor option (blinking, active)
 *
 * @param active Activate the cursor, showing it or not [TURN_ON_CURSOR, TURN_OFF_CURSOR]
 *
 * @param blinking  Let the cursor blink [BLINK_ON,BLINK_OFF]
 *
 * @note You must use the constant TURN_ON_CURSOR, TURN_OFF_CURSOR, BLINK_ON, BLINK_OFF
 *       to ensure the compatibility with LCD_44780_I2C library.
 *
 */
void CursorLCD(char active,char blinking);


/**
 * This function controls the backligth LED.
 *
 * @param active Activate the LED backlight [TURN_ON_LED, TURN_OFF_LED]
 *
 * @param blinking  Let the cursor blink [BLINK_ON,BLINK_OFF]
 *
 * @note You must use the constant TURN_ON_LED, TURN_OFF_LED
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void BacklightLCD(char active);


/**
 * This funnction initializes the LCD to work in 4 bit modality.
 *
 * @param quartz_frequency Quartz freq. expressed in MHz used to run the CPU.
 *
 *
 * @note You must properly set the microcontroller pins using the TRISx registers
 *       The library requires the delay library to be included.
 */
void  OpenLCD(unsigned char quartz_frequency);


#endif
